@@ -96,7 +96,7 @@ fn create_algorithm_from_flags(matches: &ArgMatches) -> UResult<HashAlgorithm> {
         set_or_err(detect_algo("b3sum", None)?)?;
     }
     if matches.get_flag("sha3") {
-        let bits = matches.get_one::<usize>("bits").cloned();
+        let bits = matches.get_one::<usize>("bits").copied();
         set_or_err(create_sha3(bits)?)?;
     }
     if matches.get_flag("sha3-224") {
@@ -207,16 +207,10 @@ pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
         binary_flag_default
     };
     let check = matches.get_flag("check");
-    let tag = matches.get_flag("tag");
-    let nonames = *matches
-        .try_get_one("no-names")
-        .unwrap_or(None)
-        .unwrap_or(&false);
     let status = matches.get_flag("status");
     let quiet = matches.get_flag("quiet") || status;
-    //let strict = matches.get_flag("strict");
+    let strict = matches.get_flag("strict");
     let warn = matches.get_flag("warn") && !status;
-    let zero = matches.get_flag("zero");
     let ignore_missing = matches.get_flag("ignore-missing");
 
     if ignore_missing && !check {
@@ -224,30 +218,20 @@ pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
         return Err(ChecksumError::IgnoreNotCheck.into());
     }
 
-    let opts = Options {
-        algoname: algo.name,
-        digest: (algo.create_fn)(),
-        output_bits: algo.bits,
-        binary,
-        //check,
-        tag,
-        nonames,
-        //status,
-        //quiet,
-        //strict,
-        //warn,
-        zero,
-        //ignore_missing,
-    };
-
     if check {
-        let text_flag = matches.get_flag("text");
-        let binary_flag = matches.get_flag("binary");
-        let strict = matches.get_flag("strict");
+        // on Windows, allow --binary/--text to be used with --check
+        // and keep the behavior of defaulting to binary
+        #[cfg(not(windows))]
+        let binary = {
+            let text_flag = matches.get_flag("text");
+            let binary_flag = matches.get_flag("binary");
 
-        if binary_flag || text_flag {
-            return Err(ChecksumError::BinaryTextConflict.into());
-        }
+            if binary_flag || text_flag {
+                return Err(ChecksumError::BinaryTextConflict.into());
+            }
+
+            false
+        };
 
         // Execute the checksum validation based on the presence of files or the use of stdin
         // Determine the source of input: a list of files or stdin.
@@ -262,14 +246,38 @@ pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
             strict,
             status,
             warn,
-            binary_flag,
+            binary,
             ignore_missing,
             quiet,
             Some(algo.name),
             Some(algo.bits),
         );
+    } else if quiet {
+        return Err(ChecksumError::QuietNotCheck.into());
+    } else if strict {
+        return Err(ChecksumError::StrictNotCheck.into());
     }
 
+    let nonames = *matches
+        .try_get_one("no-names")
+        .unwrap_or(None)
+        .unwrap_or(&false);
+    let zero = matches.get_flag("zero");
+
+    let opts = Options {
+        algoname: algo.name,
+        digest: (algo.create_fn)(),
+        output_bits: algo.bits,
+        binary,
+        tag: matches.get_flag("tag"),
+        nonames,
+        //status,
+        //quiet,
+        //warn,
+        zero,
+        //ignore_missing,
+    };
+
     // Show the hashsum of the input
     match matches.get_many::<OsString>(options::FILE) {
         Some(files) => hashsum(opts, files.map(|f| f.as_os_str())),
@@ -296,11 +304,11 @@ mod options {
 
 pub fn uu_app_common() -> Command {
     #[cfg(windows)]
-    const BINARY_HELP: &str = "read in binary mode (default)";
+    const BINARY_HELP: &str = "read or check in binary mode (default)";
     #[cfg(not(windows))]
     const BINARY_HELP: &str = "read in binary mode";
     #[cfg(windows)]
-    const TEXT_HELP: &str = "read in text mode";
+    const TEXT_HELP: &str = "read or check in text mode";
     #[cfg(not(windows))]
     const TEXT_HELP: &str = "read in text mode (default)";
     Command::new(uucore::util_name())
