@@ -3,12 +3,13 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore (strings) anychar combinator Alnum Punct Xdigit alnum punct xdigit cntrl boop
+// spell-checker:ignore (strings) anychar combinator Alnum Punct Xdigit alnum punct xdigit cntrl
 
+use crate::unicode_table;
 use nom::{
     branch::alt,
-    bytes::complete::{tag, take},
-    character::complete::{digit1, one_of},
+    bytes::complete::{tag, take, take_till},
+    character::complete::one_of,
     combinator::{map, map_opt, peek, recognize, value},
     multi::{many0, many_m_n},
     sequence::{delimited, preceded, separated_pair},
@@ -23,8 +24,6 @@ use std::{
 };
 use uucore::error::UError;
 
-use crate::unicode_table;
-
 #[derive(Debug, Clone)]
 pub enum BadSequence {
     MissingCharClassName,
@@ -37,6 +36,7 @@ pub enum BadSequence {
     ClassInSet2NotMatchedBySet1,
     Set1LongerSet2EndsInClass,
     ComplementMoreThanOneUniqueInSet2,
+    BackwardsRange { end: u32, start: u32 },
 }
 
 impl Display for BadSequence {
@@ -70,6 +70,23 @@ impl Display for BadSequence {
             Self::ComplementMoreThanOneUniqueInSet2 => {
                 write!(f, "when translating with complemented character classes,\nstring2 must map all characters in the domain to one")
             }
+            Self::BackwardsRange { end, start } => {
+                fn end_or_start_to_string(ut: &u32) -> String {
+                    match char::from_u32(*ut) {
+                        Some(ch @ '\x20'..='\x7E') => ch.escape_default().to_string(),
+                        _ => {
+                            format!("\\{ut:03o}")
+                        }
+                    }
+                }
+
+                write!(
+                    f,
+                    "range-endpoints of '{}-{}' are in reverse collating sequence order",
+                    end_or_start_to_string(start),
+                    end_or_start_to_string(end)
+                )
+            }
         }
     }
 }
@@ -103,22 +120,18 @@ pub enum Sequence {
 }
 
 impl Sequence {
-    pub fn flatten_non_lower_upper(&self) -> Box<dyn Iterator<Item = Self>> {
+    pub fn flatten(&self) -> Box<dyn Iterator<Item = u8>> {
         match self {
-            Self::Char(c) => Box::new(std::iter::once(*c).map(Self::Char)),
-            Self::CharRange(l, r) => Box::new((*l..=*r).map(Self::Char)),
-            Self::CharRepeat(c, n) => Box::new(std::iter::repeat(*c).take(*n).map(Self::Char)),
+            Self::Char(c) => Box::new(std::iter::once(*c)),
+            Self::CharRange(l, r) => Box::new(*l..=*r),
+            Self::CharStar(c) => Box::new(std::iter::repeat(*c)),
+            Self::CharRepeat(c, n) => Box::new(std::iter::repeat(*c).take(*n)),
             Self::Class(class) => match class {
-                Class::Alnum => Box::new(
-                    (b'0'..=b'9')
-                        .chain(b'A'..=b'Z')
-                        .chain(b'a'..=b'z')
-                        .map(Self::Char),
-                ),
-                Class::Alpha => Box::new((b'A'..=b'Z').chain(b'a'..=b'z').map(Self::Char)),
-                Class::Blank => Box::new(unicode_table::BLANK.iter().cloned().map(Self::Char)),
-                Class::Control => Box::new((0..=31).chain(std::iter::once(127)).map(Self::Char)),
-                Class::Digit => Box::new((b'0'..=b'9').map(Self::Char)),
+                Class::Alnum => Box::new((b'0'..=b'9').chain(b'A'..=b'Z').chain(b'a'..=b'z')),
+                Class::Alpha => Box::new((b'A'..=b'Z').chain(b'a'..=b'z')),
+                Class::Blank => Box::new(unicode_table::BLANK.iter().copied()),
+                Class::Control => Box::new((0..=31).chain(std::iter::once(127))),
+                Class::Digit => Box::new(b'0'..=b'9'),
                 Class::Graph => Box::new(
                     (48..=57) // digit
                         .chain(65..=90) // uppercase
@@ -128,8 +141,7 @@ impl Sequence {
                         .chain(58..=64)
                         .chain(91..=96)
                         .chain(123..=126)
-                        .chain(std::iter::once(32))
-                        .map(Self::Char), // space
+                        .chain(std::iter::once(32)), // space
                 ),
                 Class::Print => Box::new(
                     (48..=57) // digit
@@ -139,37 +151,14 @@ impl Sequence {
                         .chain(33..=47)
                         .chain(58..=64)
                         .chain(91..=96)
-                        .chain(123..=126)
-                        .map(Self::Char),
+                        .chain(123..=126),
                 ),
-                Class::Punct => Box::new(
-                    (33..=47)
-                        .chain(58..=64)
-                        .chain(91..=96)
-                        .chain(123..=126)
-                        .map(Self::Char),
-                ),
-                Class::Space => Box::new(unicode_table::SPACES.iter().cloned().map(Self::Char)),
-                Class::Xdigit => Box::new(
-                    (b'0'..=b'9')
-                        .chain(b'A'..=b'F')
-                        .chain(b'a'..=b'f')
-                        .map(Self::Char),
-                ),
-                s => Box::new(std::iter::once(Self::Class(*s))),
+                Class::Punct => Box::new((33..=47).chain(58..=64).chain(91..=96).chain(123..=126)),
+                Class::Space => Box::new(unicode_table::SPACES.iter().copied()),
+                Class::Xdigit => Box::new((b'0'..=b'9').chain(b'A'..=b'F').chain(b'a'..=b'f')),
+                Class::Lower => Box::new(b'a'..=b'z'),
+                Class::Upper => Box::new(b'A'..=b'Z'),
             },
-            s => Box::new(std::iter::once(*s)),
-        }
-    }
-
-    pub fn flatten_all(&self) -> Box<dyn Iterator<Item = Self>> {
-        match self {
-            Self::Class(class) => match class {
-                Class::Lower => Box::new((b'a'..=b'z').map(Self::Char)),
-                Class::Upper => Box::new((b'A'..=b'Z').map(Self::Char)),
-                s => Self::Class(*s).flatten_non_lower_upper(),
-            },
-            s => s.flatten_non_lower_upper(),
         }
     }
 
@@ -182,12 +171,6 @@ impl Sequence {
         translating: bool,
     ) -> Result<(Vec<u8>, Vec<u8>), BadSequence> {
         let is_char_star = |s: &&Self| -> bool { matches!(s, Self::CharStar(_)) };
-        let to_u8 = |s: Self| -> Option<u8> {
-            match s {
-                Self::Char(c) => Some(c),
-                _ => None,
-            }
-        };
 
         let set1 = Self::from_str(set1_str)?;
         if set1.iter().filter(is_char_star).count() != 0 {
@@ -208,11 +191,7 @@ impl Sequence {
             return Err(BadSequence::ClassExceptLowerUpperInSet2);
         }
 
-        let mut set1_solved: Vec<u8> = set1
-            .iter()
-            .flat_map(Self::flatten_all)
-            .filter_map(to_u8)
-            .collect();
+        let mut set1_solved: Vec<u8> = set1.iter().flat_map(Self::flatten).collect();
         if complement_flag {
             set1_solved = (0..=u8::MAX).filter(|x| !set1_solved.contains(x)).collect();
         }
@@ -224,7 +203,7 @@ impl Sequence {
                 Self::CharStar(_) => None,
                 r => Some(r),
             })
-            .flat_map(Self::flatten_all)
+            .flat_map(Self::flatten)
             .count();
 
         let star_compensate_len = set1_len.saturating_sub(set2_len);
@@ -238,45 +217,52 @@ impl Sequence {
             })
             .collect();
 
-        //Flatten everything but upper/lower into Char
-        let set1_flattened: Vec<_> = set1
-            .iter()
-            .flat_map(Self::flatten_non_lower_upper)
-            .collect();
-        set2 = set2
-            .iter()
-            .flat_map(Self::flatten_non_lower_upper)
-            .collect();
+        // For every upper/lower in set2, there must be an upper/lower in set1 at the same position. The position is calculated by expanding everything before the upper/lower in both sets
+        for (set2_pos, set2_item) in set2.iter().enumerate() {
+            if matches!(set2_item, Self::Class(_)) {
+                let mut set2_part_solved_len = 0;
+                if set2_pos >= 1 {
+                    set2_part_solved_len =
+                        set2.iter().take(set2_pos).flat_map(Self::flatten).count();
+                }
 
-        if set2
-            .iter()
-            .zip(
-                set1_flattened
-                    .iter()
-                    .chain(std::iter::repeat(&Self::Char(0))),
-            )
-            .any(|x| matches!(x.0, Self::Class(_)) && !matches!(x.1, Self::Class(_)))
-        {
-            return Err(BadSequence::ClassInSet2NotMatchedBySet1);
+                let mut class_matches = false;
+                for (set1_pos, set1_item) in set1.iter().enumerate() {
+                    if matches!(set1_item, Self::Class(_)) {
+                        let mut set1_part_solved_len = 0;
+                        if set1_pos >= 1 {
+                            set1_part_solved_len =
+                                set1.iter().take(set1_pos).flat_map(Self::flatten).count();
+                        }
+
+                        if set1_part_solved_len == set2_part_solved_len {
+                            class_matches = true;
+                            break;
+                        }
+                    }
+                }
+
+                if !class_matches {
+                    return Err(BadSequence::ClassInSet2NotMatchedBySet1);
+                }
+            }
         }
 
-        let set2_solved: Vec<_> = set2
-            .iter()
-            .flat_map(Self::flatten_all)
-            .filter_map(to_u8)
-            .collect();
+        let set2_solved: Vec<_> = set2.iter().flat_map(Self::flatten).collect();
 
         // Calculate the set of unique characters in set2
         let mut set2_uniques = set2_solved.clone();
         set2_uniques.sort();
         set2_uniques.dedup();
 
-        //If the complement flag is used in translate mode, only one unique character may appear in
-        //set2. Validate this with the set of uniques in set2 that we just generated.
+        // If the complement flag is used in translate mode, only one unique
+        // character may appear in set2. Validate this with the set of uniques
+        // in set2 that we just generated.
+        // Also, set2 must not overgrow set1, otherwise the mapping can't be 1:1.
         if set1.iter().any(|x| matches!(x, Self::Class(_)))
             && translating
             && complement_flag
-            && set2_uniques.len() > 1
+            && (set2_uniques.len() > 1 || set2_solved.len() > set1_len)
         {
             return Err(BadSequence::ComplementMoreThanOneUniqueInSet2);
         }
@@ -316,9 +302,41 @@ impl Sequence {
     }
 
     fn parse_octal(input: &[u8]) -> IResult<&[u8], u8> {
+        preceded(
+            tag("\\"),
+            alt((
+                Self::parse_octal_up_to_three_digits,
+                // Fallback for if the three digit octal escape is greater than \377 (0xFF), and therefore can't be
+                // parsed as as a byte
+                // See test `test_multibyte_octal_sequence`
+                Self::parse_octal_two_digits,
+            )),
+        )(input)
+    }
+
+    fn parse_octal_up_to_three_digits(input: &[u8]) -> IResult<&[u8], u8> {
         map_opt(
-            preceded(tag("\\"), recognize(many_m_n(1, 3, one_of("01234567")))),
-            |out: &[u8]| u8::from_str_radix(std::str::from_utf8(out).expect("boop"), 8).ok(),
+            recognize(many_m_n(1, 3, one_of("01234567"))),
+            |out: &[u8]| {
+                let str_to_parse = std::str::from_utf8(out).unwrap();
+
+                match u8::from_str_radix(str_to_parse, 8) {
+                    Ok(ue) => Some(ue),
+                    Err(_pa) => {
+                        // TODO
+                        // A warning needs to be printed here
+                        // See https://github.com/uutils/coreutils/issues/6821
+                        None
+                    }
+                }
+            },
+        )(input)
+    }
+
+    fn parse_octal_two_digits(input: &[u8]) -> IResult<&[u8], u8> {
+        map_opt(
+            recognize(many_m_n(2, 2, one_of("01234567"))),
+            |out: &[u8]| u8::from_str_radix(std::str::from_utf8(out).unwrap(), 8).ok(),
         )(input)
     }
 
@@ -355,7 +373,14 @@ impl Sequence {
         .map(|(l, (a, b))| {
             (l, {
                 let (start, end) = (u32::from(a), u32::from(b));
-                Ok(Self::CharRange(start as u8, end as u8))
+
+                let range = start..=end;
+
+                if range.is_empty() {
+                    Err(BadSequence::BackwardsRange { end, start })
+                } else {
+                    Ok(Self::CharRange(start as u8, end as u8))
+                }
             })
         })
     }
@@ -368,7 +393,14 @@ impl Sequence {
     fn parse_char_repeat(input: &[u8]) -> IResult<&[u8], Result<Self, BadSequence>> {
         delimited(
             tag("["),
-            separated_pair(Self::parse_backslash_or_char, tag("*"), digit1),
+            separated_pair(
+                Self::parse_backslash_or_char,
+                tag("*"),
+                // TODO
+                // Why are the opening and closing tags not sufficient?
+                // Backslash check is a workaround for `check_against_gnu_tr_tests_repeat_bs_9`
+                take_till(|ue| matches!(ue, b']' | b'\\')),
+            ),
             tag("]"),
         )(input)
         .map(|(l, (c, cnt_str))| {
