@@ -5,6 +5,7 @@
 
 //! Utilities for formatting numbers in various formats
 
+use std::cmp::min;
 use std::io::Write;
 
 use super::{
@@ -77,22 +78,16 @@ pub struct SignedInt {
 impl Formatter for SignedInt {
     type Input = i64;
 
-    fn fmt(&self, mut writer: impl Write, x: Self::Input) -> std::io::Result<()> {
-        if x >= 0 {
-            match self.positive_sign {
-                PositiveSign::None => Ok(()),
-                PositiveSign::Plus => write!(writer, "+"),
-                PositiveSign::Space => write!(writer, " "),
-            }?;
-        }
+    fn fmt(&self, writer: impl Write, x: Self::Input) -> std::io::Result<()> {
+        let s = if self.precision > 0 {
+            format!("{:0>width$}", x.abs(), width = self.precision)
+        } else {
+            x.abs().to_string()
+        };
 
-        let s = format!("{:0width$}", x, width = self.precision);
+        let sign_indicator = get_sign_indicator(self.positive_sign, &x);
 
-        match self.alignment {
-            NumberAlignment::Left => write!(writer, "{s:<width$}", width = self.width),
-            NumberAlignment::RightSpace => write!(writer, "{s:>width$}", width = self.width),
-            NumberAlignment::RightZero => write!(writer, "{s:0>width$}", width = self.width),
-        }
+        write_output(writer, sign_indicator, s, self.width, self.alignment)
     }
 
     fn try_from_spec(s: Spec) -> Result<Self, FormatError> {
@@ -209,9 +204,9 @@ impl Formatter for UnsignedInt {
         };
 
         Ok(Self {
+            variant,
             width,
             precision,
-            variant,
             alignment,
         })
     }
@@ -244,16 +239,8 @@ impl Default for Float {
 impl Formatter for Float {
     type Input = f64;
 
-    fn fmt(&self, mut writer: impl Write, x: Self::Input) -> std::io::Result<()> {
-        if x.is_sign_positive() {
-            match self.positive_sign {
-                PositiveSign::None => Ok(()),
-                PositiveSign::Plus => write!(writer, "+"),
-                PositiveSign::Space => write!(writer, " "),
-            }?;
-        }
-
-        let s = if x.is_finite() {
+    fn fmt(&self, writer: impl Write, x: Self::Input) -> std::io::Result<()> {
+        let mut s = if x.is_finite() {
             match self.variant {
                 FloatVariant::Decimal => {
                     format_float_decimal(x, self.precision, self.force_decimal)
@@ -272,11 +259,13 @@ impl Formatter for Float {
             format_float_non_finite(x, self.case)
         };
 
-        match self.alignment {
-            NumberAlignment::Left => write!(writer, "{s:<width$}", width = self.width),
-            NumberAlignment::RightSpace => write!(writer, "{s:>width$}", width = self.width),
-            NumberAlignment::RightZero => write!(writer, "{s:0>width$}", width = self.width),
-        }
+        // The format function will parse `x` together with its sign char,
+        // which should be placed in `sign_indicator`. So drop it here
+        s = if x < 0. { s[1..].to_string() } else { s };
+
+        let sign_indicator = get_sign_indicator(self.positive_sign, &x);
+
+        write_output(writer, sign_indicator, s, self.width, self.alignment)
     }
 
     fn try_from_spec(s: Spec) -> Result<Self, FormatError>
@@ -326,6 +315,18 @@ impl Formatter for Float {
     }
 }
 
+fn get_sign_indicator<T: PartialOrd + Default>(sign: PositiveSign, x: &T) -> String {
+    if *x >= T::default() {
+        match sign {
+            PositiveSign::None => String::new(),
+            PositiveSign::Plus => String::from("+"),
+            PositiveSign::Space => String::from(" "),
+        }
+    } else {
+        String::from("-")
+    }
+}
+
 fn format_float_non_finite(f: f64, case: Case) -> String {
     debug_assert!(!f.is_finite());
     let mut s = format!("{f}");
@@ -339,7 +340,7 @@ fn format_float_decimal(f: f64, precision: usize, force_decimal: ForceDecimal) -
     if precision == 0 && force_decimal == ForceDecimal::Yes {
         format!("{f:.0}.")
     } else {
-        format!("{f:.*}", precision)
+        format!("{f:.precision$}")
     }
 }
 
@@ -379,10 +380,7 @@ fn format_float_scientific(
         Case::Uppercase => 'E',
     };
 
-    format!(
-        "{normalized:.*}{additional_dot}{exp_char}{exponent:+03}",
-        precision
-    )
+    format!("{normalized:.precision$}{additional_dot}{exp_char}{exponent:+03}")
 }
 
 fn format_float_shortest(
@@ -426,7 +424,7 @@ fn format_float_shortest(
             ""
         };
 
-        let mut normalized = format!("{normalized:.*}", precision);
+        let mut normalized = format!("{normalized:.precision$}");
 
         if force_decimal == ForceDecimal::No {
             strip_fractional_zeroes_and_dot(&mut normalized);
@@ -448,7 +446,7 @@ fn format_float_shortest(
         let mut formatted = if decimal_places == 0 && force_decimal == ForceDecimal::Yes {
             format!("{f:.0}.")
         } else {
-            format!("{f:.*}", decimal_places)
+            format!("{f:.decimal_places$}")
         };
 
         if force_decimal == ForceDecimal::No {
@@ -501,6 +499,35 @@ fn strip_fractional_zeroes_and_dot(s: &mut String) {
     }
 }
 
+fn write_output(
+    mut writer: impl Write,
+    sign_indicator: String,
+    mut s: String,
+    width: usize,
+    alignment: NumberAlignment,
+) -> std::io::Result<()> {
+    // Take length of `sign_indicator`, which could be 0 or 1, into consideration when padding
+    // by storing remaining_width indicating the actual width needed.
+    // Using min() because self.width could be 0, 0usize - 1usize should be avoided
+    let remaining_width = width - min(width, sign_indicator.len());
+    match alignment {
+        NumberAlignment::Left => write!(writer, "{sign_indicator}{s:<remaining_width$}"),
+        NumberAlignment::RightSpace => {
+            let is_sign = sign_indicator.starts_with('-') || sign_indicator.starts_with('+'); // When sign_indicator is in ['-', '+']
+            if is_sign && remaining_width > 0 {
+                // Make sure sign_indicator is just next to number, e.g. "% +5.1f" 1 ==> $ +1.0
+                s = sign_indicator + s.as_str();
+                write!(writer, "{s:>width$}", width = remaining_width + 1) // Since we now add sign_indicator and s together, plus 1
+            } else {
+                write!(writer, "{sign_indicator}{s:>remaining_width$}")
+            }
+        }
+        NumberAlignment::RightZero => {
+            write!(writer, "{sign_indicator}{s:0>remaining_width$}")
+        }
+    }
+}
+
 #[cfg(test)]
 mod test {
     use crate::format::num_format::{Case, ForceDecimal};
@@ -533,12 +560,12 @@ mod test {
         assert_eq!(f(0.0), "0.000000");
         assert_eq!(f(1.0), "1.000000");
         assert_eq!(f(100.0), "100.000000");
-        assert_eq!(f(123456.789), "123456.789000");
-        assert_eq!(f(12.3456789), "12.345679");
-        assert_eq!(f(1000000.0), "1000000.000000");
-        assert_eq!(f(99999999.0), "99999999.000000");
-        assert_eq!(f(1.9999995), "1.999999");
-        assert_eq!(f(1.9999996), "2.000000");
+        assert_eq!(f(123_456.789), "123456.789000");
+        assert_eq!(f(12.345_678_9), "12.345679");
+        assert_eq!(f(1_000_000.0), "1000000.000000");
+        assert_eq!(f(99_999_999.0), "99999999.000000");
+        assert_eq!(f(1.999_999_5), "1.999999");
+        assert_eq!(f(1.999_999_6), "2.000000");
     }
 
     #[test]
@@ -548,10 +575,10 @@ mod test {
         assert_eq!(f(0.0), "0.000000e+00");
         assert_eq!(f(1.0), "1.000000e+00");
         assert_eq!(f(100.0), "1.000000e+02");
-        assert_eq!(f(123456.789), "1.234568e+05");
-        assert_eq!(f(12.3456789), "1.234568e+01");
-        assert_eq!(f(1000000.0), "1.000000e+06");
-        assert_eq!(f(99999999.0), "1.000000e+08");
+        assert_eq!(f(123_456.789), "1.234568e+05");
+        assert_eq!(f(12.345_678_9), "1.234568e+01");
+        assert_eq!(f(1_000_000.0), "1.000000e+06");
+        assert_eq!(f(99_999_999.0), "1.000000e+08");
     }
 
     #[test]
@@ -562,19 +589,19 @@ mod test {
         assert_eq!(f(0.0), "0e+00");
         assert_eq!(f(1.0), "1e+00");
         assert_eq!(f(100.0), "1e+02");
-        assert_eq!(f(123456.789), "1e+05");
-        assert_eq!(f(12.3456789), "1e+01");
-        assert_eq!(f(1000000.0), "1e+06");
-        assert_eq!(f(99999999.0), "1e+08");
+        assert_eq!(f(123_456.789), "1e+05");
+        assert_eq!(f(12.345_678_9), "1e+01");
+        assert_eq!(f(1_000_000.0), "1e+06");
+        assert_eq!(f(99_999_999.0), "1e+08");
 
         let f = |x| format_float_scientific(x, 0, Case::Lowercase, ForceDecimal::Yes);
         assert_eq!(f(0.0), "0.e+00");
         assert_eq!(f(1.0), "1.e+00");
         assert_eq!(f(100.0), "1.e+02");
-        assert_eq!(f(123456.789), "1.e+05");
-        assert_eq!(f(12.3456789), "1.e+01");
-        assert_eq!(f(1000000.0), "1.e+06");
-        assert_eq!(f(99999999.0), "1.e+08");
+        assert_eq!(f(123_456.789), "1.e+05");
+        assert_eq!(f(12.345_678_9), "1.e+01");
+        assert_eq!(f(1_000_000.0), "1.e+06");
+        assert_eq!(f(99_999_999.0), "1.e+08");
     }
 
     #[test]
@@ -584,10 +611,10 @@ mod test {
         assert_eq!(f(0.0), "0");
         assert_eq!(f(1.0), "1");
         assert_eq!(f(100.0), "100");
-        assert_eq!(f(123456.789), "123457");
-        assert_eq!(f(12.3456789), "12.3457");
-        assert_eq!(f(1000000.0), "1e+06");
-        assert_eq!(f(99999999.0), "1e+08");
+        assert_eq!(f(123_456.789), "123457");
+        assert_eq!(f(12.345_678_9), "12.3457");
+        assert_eq!(f(1_000_000.0), "1e+06");
+        assert_eq!(f(99_999_999.0), "1e+08");
     }
 
     #[test]
@@ -597,10 +624,10 @@ mod test {
         assert_eq!(f(0.0), "0.00000");
         assert_eq!(f(1.0), "1.00000");
         assert_eq!(f(100.0), "100.000");
-        assert_eq!(f(123456.789), "123457.");
-        assert_eq!(f(12.3456789), "12.3457");
-        assert_eq!(f(1000000.0), "1.00000e+06");
-        assert_eq!(f(99999999.0), "1.00000e+08");
+        assert_eq!(f(123_456.789), "123457.");
+        assert_eq!(f(12.345_678_9), "12.3457");
+        assert_eq!(f(1_000_000.0), "1.00000e+06");
+        assert_eq!(f(99_999_999.0), "1.00000e+08");
     }
 
     #[test]
@@ -610,19 +637,19 @@ mod test {
         assert_eq!(f(0.0), "0");
         assert_eq!(f(1.0), "1");
         assert_eq!(f(100.0), "1e+02");
-        assert_eq!(f(123456.789), "1e+05");
-        assert_eq!(f(12.3456789), "1e+01");
-        assert_eq!(f(1000000.0), "1e+06");
-        assert_eq!(f(99999999.0), "1e+08");
+        assert_eq!(f(123_456.789), "1e+05");
+        assert_eq!(f(12.345_678_9), "1e+01");
+        assert_eq!(f(1_000_000.0), "1e+06");
+        assert_eq!(f(99_999_999.0), "1e+08");
 
         let f = |x| format_float_shortest(x, 0, Case::Lowercase, ForceDecimal::Yes);
         assert_eq!(f(0.0), "0.");
         assert_eq!(f(1.0), "1.");
         assert_eq!(f(100.0), "1.e+02");
-        assert_eq!(f(123456.789), "1.e+05");
-        assert_eq!(f(12.3456789), "1.e+01");
-        assert_eq!(f(1000000.0), "1.e+06");
-        assert_eq!(f(99999999.0), "1.e+08");
+        assert_eq!(f(123_456.789), "1.e+05");
+        assert_eq!(f(12.345_678_9), "1.e+01");
+        assert_eq!(f(1_000_000.0), "1.e+06");
+        assert_eq!(f(99_999_999.0), "1.e+08");
     }
 
     #[test]
