@@ -117,6 +117,17 @@ fn test_whitespace_with_char() {
         .code_is(1);
 }
 
+#[test]
+fn test_delimiter_with_byte_and_char() {
+    for conflicting_arg in ["-c", "-b"] {
+        new_ucmd!()
+            .args(&[conflicting_arg, COMPLEX_SEQUENCE.sequence, "-d="])
+            .fails()
+            .stderr_is("cut: invalid input: The '--delimiter' ('-d') option only usable if printing a sequence of fields\n")
+            .code_is(1);
+    }
+}
+
 #[test]
 fn test_too_large() {
     new_ucmd!()
@@ -289,6 +300,13 @@ fn test_multiple_mode_args() {
     }
 }
 
+#[test]
+fn test_no_argument() {
+    new_ucmd!().fails().stderr_is(
+        "cut: invalid usage: expects one of --fields (-f), --chars (-c) or --bytes (-b)\n",
+    );
+}
+
 #[test]
 #[cfg(unix)]
 fn test_8bit_non_utf8_delimiter() {
