@@ -2,11 +2,13 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore xzaaa sixhundredfiftyonebytes ninetyonebytes threebytes asciilowercase fghij klmno pqrst uvwxyz fivelines twohundredfortyonebytes onehundredlines nbbbb dxen ncccc
+// spell-checker:ignore xzaaa sixhundredfiftyonebytes ninetyonebytes threebytes asciilowercase ghijkl mnopq rstuv wxyz fivelines twohundredfortyonebytes onehundredlines nbbbb dxen ncccc rlimit NOFILE
 
 use crate::common::util::{AtPath, TestScenario};
 use rand::{thread_rng, Rng, SeedableRng};
 use regex::Regex;
+#[cfg(any(target_os = "linux", target_os = "android"))]
+use rlimit::Resource;
 #[cfg(not(windows))]
 use std::env;
 use std::path::Path;
@@ -121,6 +123,15 @@ fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
 }
 
+#[test]
+fn test_split_non_existing_file() {
+    new_ucmd!()
+        .arg("non-existing")
+        .fails()
+        .code_is(1)
+        .stderr_is("split: cannot open 'non-existing' for reading: No such file or directory\n");
+}
+
 #[test]
 fn test_split_default() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -336,6 +347,7 @@ fn test_filter_command_fails() {
 
 #[test]
 #[cfg(unix)]
+#[cfg(not(target_os = "openbsd"))]
 fn test_filter_broken_pipe() {
     let (at, mut ucmd) = at_and_ucmd!();
     let name = "filter-big-input";
@@ -606,13 +618,13 @@ fn test_split_obs_lines_as_other_option_value() {
         .args(&["-n", "-200", "file"])
         .fails()
         .code_is(1)
-        .stderr_contains("split: invalid number of chunks: -200\n");
+        .stderr_contains("split: invalid number of chunks: '-200'\n");
     scene
         .ucmd()
         .args(&["--number", "-e200", "file"])
         .fails()
         .code_is(1)
-        .stderr_contains("split: invalid number of chunks: -e200\n");
+        .stderr_contains("split: invalid number of chunks: '-e200'\n");
 }
 
 /// Test for using more than one obsolete lines option (standalone)
@@ -694,7 +706,6 @@ fn test_split_invalid_bytes_size() {
 
 #[test]
 fn test_split_overflow_bytes_size() {
-    #[cfg(not(target_pointer_width = "128"))]
     let (at, mut ucmd) = at_and_ucmd!();
     let name = "test_split_overflow_bytes_size";
     RandomFile::new(&at, name).add_bytes(1000);
@@ -704,60 +715,41 @@ fn test_split_overflow_bytes_size() {
     assert_eq!(glob.collate(), at.read_bytes(name));
 }
 
-#[test]
-#[cfg(target_pointer_width = "32")]
-fn test_split_chunks_num_chunks_oversized_32() {
-    let scene = TestScenario::new(util_name!());
-    let at = &scene.fixtures;
-    at.touch("file");
-    scene
-        .ucmd()
-        .args(&["--number", "5000000000", "sixhundredfiftyonebytes.txt"])
-        .fails()
-        .code_is(1)
-        .stderr_only("split: Number of chunks too big\n");
-}
-
 #[test]
 fn test_split_stdin_num_chunks() {
-    new_ucmd!()
-        .args(&["--number=1"])
-        .fails()
-        .code_is(1)
-        .stderr_only("split: -: cannot determine file size\n");
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["--number=1"]).pipe_in("").succeeds();
+    assert_eq!(at.read("xaa"), "");
+    assert!(!at.plus("xab").exists());
 }
 
 #[test]
 fn test_split_stdin_num_kth_chunk() {
     new_ucmd!()
         .args(&["--number=1/2"])
-        .fails()
-        .code_is(1)
-        .stderr_only("split: -: cannot determine file size\n");
+        .pipe_in("1\n2\n3\n4\n5\n")
+        .succeeds()
+        .stdout_only("1\n2\n3");
 }
 
 #[test]
 fn test_split_stdin_num_line_chunks() {
-    new_ucmd!()
-        .args(&["--number=l/2"])
-        .fails()
-        .code_is(1)
-        .stderr_only("split: -: cannot determine file size\n");
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["--number=l/2"])
+        .pipe_in("1\n2\n3\n4\n5\n")
+        .succeeds();
+    assert_eq!(at.read("xaa"), "1\n2\n3\n");
+    assert_eq!(at.read("xab"), "4\n5\n");
+    assert!(!at.plus("xac").exists());
 }
 
 #[test]
 fn test_split_stdin_num_kth_line_chunk() {
     new_ucmd!()
         .args(&["--number=l/2/5"])
-        .fails()
-        .code_is(1)
-        .stderr_only("split: -: cannot determine file size\n");
-}
-
-fn file_read(at: &AtPath, filename: &str) -> String {
-    let mut s = String::new();
-    at.open(filename).read_to_string(&mut s).unwrap();
-    s
+        .pipe_in("1\n2\n3\n4\n5\n")
+        .succeeds()
+        .stdout_only("2\n");
 }
 
 /// Test for the default suffix length behavior: dynamically increasing size.
@@ -779,11 +771,11 @@ fn test_alphabetic_dynamic_suffix_length() {
     for i in b'a'..=b'y' {
         for j in b'a'..=b'z' {
             let filename = format!("x{}{}", i as char, j as char);
-            let contents = file_read(&at, &filename);
+            let contents = at.read(&filename);
             assert_eq!(contents, "a");
         }
     }
-    assert_eq!(file_read(&at, "xzaaa"), "a");
+    assert_eq!(at.read("xzaaa"), "a");
 }
 
 /// Test for the default suffix length behavior: dynamically increasing size.
@@ -803,10 +795,10 @@ fn test_numeric_dynamic_suffix_length() {
         .succeeds();
     for i in 0..90 {
         let filename = format!("x{i:02}");
-        let contents = file_read(&at, &filename);
+        let contents = at.read(&filename);
         assert_eq!(contents, "a");
     }
-    assert_eq!(file_read(&at, "x9000"), "a");
+    assert_eq!(at.read("x9000"), "a");
 }
 
 #[test]
@@ -825,10 +817,10 @@ fn test_hex_dynamic_suffix_length() {
         .succeeds();
     for i in 0..240 {
         let filename = format!("x{i:02x}");
-        let contents = file_read(&at, &filename);
+        let contents = at.read(&filename);
         assert_eq!(contents, "a");
     }
-    assert_eq!(file_read(&at, "xf000"), "a");
+    assert_eq!(at.read("xf000"), "a");
 }
 
 /// Test for dynamic suffix length (auto-widening) disabled when suffix start number is specified
@@ -846,7 +838,7 @@ fn test_dynamic_suffix_length_on_with_suffix_start_no_value() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["-b", "1", "--numeric-suffixes", "ninetyonebytes.txt"])
         .succeeds();
-    assert_eq!(file_read(&at, "x9000"), "a");
+    assert_eq!(at.read("x9000"), "a");
 }
 
 /// Test for suffix auto-width with --number strategy and suffix start number
@@ -858,8 +850,8 @@ fn test_suffix_auto_width_with_number() {
     let glob = Glob::new(&at, ".", r"x\d\d\d$");
     assert_eq!(glob.count(), 100);
     assert_eq!(glob.collate(), at.read_bytes("fivelines.txt"));
-    assert_eq!(file_read(&at, "x001"), "1\n");
-    assert_eq!(file_read(&at, "x100"), "");
+    assert_eq!(at.read("x001"), "1\n");
+    assert_eq!(at.read("x100"), "");
 
     new_ucmd!()
         .args(&["--numeric-suffixes=100", "--number=r/100", "fivelines.txt"])
@@ -912,6 +904,14 @@ fn test_suffixes_exhausted() {
         .stderr_only("split: output file suffixes exhausted\n");
 }
 
+#[test]
+fn test_suffix_length_req() {
+    new_ucmd!()
+        .args(&["-n", "100", "-a", "1", "asciilowercase.txt"])
+        .fails()
+        .stderr_only("split: the suffix length needs to be at least 2\n");
+}
+
 #[test]
 fn test_verbose() {
     new_ucmd!()
@@ -931,17 +931,12 @@ creating file 'xaf'
 #[test]
 fn test_number_n() {
     let (at, mut ucmd) = at_and_ucmd!();
-    let file_read = |f| {
-        let mut s = String::new();
-        at.open(f).read_to_string(&mut s).unwrap();
-        s
-    };
     ucmd.args(&["-n", "5", "asciilowercase.txt"]).succeeds();
-    assert_eq!(file_read("xaa"), "abcde");
-    assert_eq!(file_read("xab"), "fghij");
-    assert_eq!(file_read("xac"), "klmno");
-    assert_eq!(file_read("xad"), "pqrst");
-    assert_eq!(file_read("xae"), "uvwxyz\n");
+    assert_eq!(at.read("xaa"), "abcdef");
+    assert_eq!(at.read("xab"), "ghijkl");
+    assert_eq!(at.read("xac"), "mnopq");
+    assert_eq!(at.read("xad"), "rstuv");
+    assert_eq!(at.read("xae"), "wxyz\n");
     #[cfg(unix)]
     new_ucmd!()
         .args(&["--number=100", "/dev/null"])
@@ -954,11 +949,11 @@ fn test_number_kth_of_n() {
     new_ucmd!()
         .args(&["--number=3/5", "asciilowercase.txt"])
         .succeeds()
-        .stdout_only("klmno");
+        .stdout_only("mnopq");
     new_ucmd!()
         .args(&["--number=5/5", "asciilowercase.txt"])
         .succeeds()
-        .stdout_only("uvwxyz\n");
+        .stdout_only("wxyz\n");
     new_ucmd!()
         .args(&["-e", "--number=99/100", "asciilowercase.txt"])
         .succeeds()
@@ -979,11 +974,11 @@ fn test_number_kth_of_n() {
     new_ucmd!()
         .args(&["--number=0/5", "asciilowercase.txt"])
         .fails()
-        .stderr_contains("split: invalid chunk number: 0");
+        .stderr_contains("split: invalid chunk number: '0'");
     new_ucmd!()
         .args(&["--number=10/5", "asciilowercase.txt"])
         .fails()
-        .stderr_contains("split: invalid chunk number: 10");
+        .stderr_contains("split: invalid chunk number: '10'");
     #[cfg(target_pointer_width = "64")]
     new_ucmd!()
         .args(&[
@@ -991,7 +986,7 @@ fn test_number_kth_of_n() {
             "asciilowercase.txt",
         ])
         .fails()
-        .stderr_contains("split: invalid number of chunks: 18446744073709551616");
+        .stderr_contains("split: invalid number of chunks: '18446744073709551616'");
 }
 
 #[test]
@@ -1025,32 +1020,27 @@ fn test_number_kth_of_n_round_robin() {
             "fivelines.txt",
         ])
         .fails()
-        .stderr_contains("split: invalid number of chunks: 18446744073709551616");
+        .stderr_contains("split: invalid number of chunks: '18446744073709551616'");
     new_ucmd!()
         .args(&["--number", "r/0/3", "fivelines.txt"])
         .fails()
-        .stderr_contains("split: invalid chunk number: 0");
+        .stderr_contains("split: invalid chunk number: '0'");
     new_ucmd!()
         .args(&["--number", "r/10/3", "fivelines.txt"])
         .fails()
-        .stderr_contains("split: invalid chunk number: 10");
+        .stderr_contains("split: invalid chunk number: '10'");
 }
 
 #[test]
 fn test_split_number_with_io_blksize() {
     let (at, mut ucmd) = at_and_ucmd!();
-    let file_read = |f| {
-        let mut s = String::new();
-        at.open(f).read_to_string(&mut s).unwrap();
-        s
-    };
     ucmd.args(&["-n", "5", "asciilowercase.txt", "---io-blksize", "1024"])
         .succeeds();
-    assert_eq!(file_read("xaa"), "abcde");
-    assert_eq!(file_read("xab"), "fghij");
-    assert_eq!(file_read("xac"), "klmno");
-    assert_eq!(file_read("xad"), "pqrst");
-    assert_eq!(file_read("xae"), "uvwxyz\n");
+    assert_eq!(at.read("xaa"), "abcdef");
+    assert_eq!(at.read("xab"), "ghijkl");
+    assert_eq!(at.read("xac"), "mnopq");
+    assert_eq!(at.read("xad"), "rstuv");
+    assert_eq!(at.read("xae"), "wxyz\n");
 }
 
 #[test]
@@ -1065,6 +1055,33 @@ fn test_split_default_with_io_blksize() {
     assert_eq!(glob.collate(), at.read_bytes(name));
 }
 
+#[test]
+fn test_split_invalid_io_blksize() {
+    new_ucmd!()
+        .args(&["---io-blksize=XYZ", "threebytes.txt"])
+        .fails()
+        .stderr_only("split: invalid IO block size: 'XYZ'\n");
+    new_ucmd!()
+        .args(&["---io-blksize=5000000000", "threebytes.txt"])
+        .fails()
+        .stderr_only("split: invalid IO block size: '5000000000'\n");
+    #[cfg(target_pointer_width = "32")]
+    new_ucmd!()
+        .args(&["---io-blksize=2146435072", "threebytes.txt"])
+        .fails()
+        .stderr_only("split: invalid IO block size: '2146435072'\n");
+}
+
+#[test]
+fn test_split_number_oversized_stdin() {
+    new_ucmd!()
+        .args(&["--number=3", "---io-blksize=600"])
+        .pipe_in_fixture("sixhundredfiftyonebytes.txt")
+        .ignore_stdin_write_error()
+        .fails()
+        .stderr_only("split: -: cannot determine input size\n");
+}
+
 #[test]
 fn test_invalid_suffix_length() {
     new_ucmd!()
@@ -1132,7 +1149,7 @@ fn test_allow_empty_files() {
 }
 
 #[test]
-fn test_elide_empty_files() {
+fn test_elide_empty_files_n_chunks() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["-e", "-n", "4", "threebytes.txt"])
         .succeeds()
@@ -1146,7 +1163,7 @@ fn test_elide_empty_files() {
 
 #[test]
 #[cfg(unix)]
-fn test_elide_dev_null() {
+fn test_elide_dev_null_n_chunks() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["-e", "-n", "3", "/dev/null"])
         .succeeds()
@@ -1158,30 +1175,108 @@ fn test_elide_dev_null() {
 }
 
 #[test]
-fn test_lines() {
+#[cfg(unix)]
+fn test_dev_zero() {
     let (at, mut ucmd) = at_and_ucmd!();
-
-    let file_read = |f| {
-        let mut s = String::new();
-        at.open(f).read_to_string(&mut s).unwrap();
-        s
-    };
-
-    // Split into two files without splitting up lines.
-    ucmd.args(&["-n", "l/2", "fivelines.txt"]).succeeds();
-
-    assert_eq!(file_read("xaa"), "1\n2\n3\n");
-    assert_eq!(file_read("xab"), "4\n5\n");
+    ucmd.args(&["-n", "3", "/dev/zero"])
+        .fails()
+        .stderr_only("split: /dev/zero: cannot determine file size\n");
+    assert!(!at.plus("xaa").exists());
+    assert!(!at.plus("xab").exists());
+    assert!(!at.plus("xac").exists());
 }
 
 #[test]
-fn test_lines_kth() {
+fn test_elide_empty_files_l_chunks() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["-e", "-n", "l/7", "fivelines.txt"])
+        .succeeds()
+        .no_stdout()
+        .no_stderr();
+    assert_eq!(at.read("xaa"), "1\n");
+    assert_eq!(at.read("xab"), "2\n");
+    assert_eq!(at.read("xac"), "3\n");
+    assert_eq!(at.read("xad"), "4\n");
+    assert_eq!(at.read("xae"), "5\n");
+    assert!(!at.plus("xaf").exists());
+    assert!(!at.plus("xag").exists());
+}
+
+#[test]
+#[cfg(unix)]
+fn test_elide_dev_null_l_chunks() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["-e", "-n", "l/3", "/dev/null"])
+        .succeeds()
+        .no_stdout()
+        .no_stderr();
+    assert!(!at.plus("xaa").exists());
+    assert!(!at.plus("xab").exists());
+    assert!(!at.plus("xac").exists());
+}
+
+#[test]
+#[cfg(unix)]
+fn test_number_by_bytes_dev_zero() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["-n", "3", "/dev/zero"])
+        .fails()
+        .stderr_only("split: /dev/zero: cannot determine file size\n");
+    assert!(!at.plus("xaa").exists());
+    assert!(!at.plus("xab").exists());
+    assert!(!at.plus("xac").exists());
+}
+
+#[test]
+fn test_number_by_lines() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    // Split into two files without splitting up lines.
+    ucmd.args(&["-n", "l/2", "fivelines.txt"]).succeeds();
+
+    assert_eq!(at.read("xaa"), "1\n2\n3\n");
+    assert_eq!(at.read("xab"), "4\n5\n");
+}
+
+#[test]
+fn test_number_by_lines_kth() {
     new_ucmd!()
         .args(&["-n", "l/3/10", "onehundredlines.txt"])
         .succeeds()
         .stdout_only("20\n21\n22\n23\n24\n25\n26\n27\n28\n29\n");
 }
 
+#[test]
+#[cfg(unix)]
+fn test_number_by_lines_kth_dev_null() {
+    new_ucmd!()
+        .args(&["-n", "l/3/10", "/dev/null"])
+        .succeeds()
+        .stdout_only("");
+}
+
+#[test]
+fn test_number_by_lines_kth_no_end_sep() {
+    new_ucmd!()
+        .args(&["-n", "l/3/10"])
+        .pipe_in("1\n2222\n3\n4")
+        .succeeds()
+        .stdout_only("2222\n");
+    new_ucmd!()
+        .args(&["-e", "-n", "l/2/2"])
+        .pipe_in("1\n2222\n3\n4")
+        .succeeds()
+        .stdout_only("3\n4");
+}
+
+#[test]
+fn test_number_by_lines_rr_kth_no_end_sep() {
+    new_ucmd!()
+        .args(&["-n", "r/2/3"])
+        .pipe_in("1\n2\n3\n4\n5")
+        .succeeds()
+        .stdout_only("2\n5");
+}
+
 #[test]
 fn test_line_bytes() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -1321,7 +1416,7 @@ fn test_numeric_suffix() {
 }
 
 #[test]
-fn test_numeric_suffix_alias() {
+fn test_numeric_suffix_inferred() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["-n", "4", "--numeric=9", "threebytes.txt"])
         .succeeds()
@@ -1546,17 +1641,19 @@ fn test_effective_suffix_hex_last() {
 #[test]
 fn test_round_robin() {
     let (at, mut ucmd) = at_and_ucmd!();
-
-    let file_read = |f| {
-        let mut s = String::new();
-        at.open(f).read_to_string(&mut s).unwrap();
-        s
-    };
-
     ucmd.args(&["-n", "r/2", "fivelines.txt"]).succeeds();
 
-    assert_eq!(file_read("xaa"), "1\n3\n5\n");
-    assert_eq!(file_read("xab"), "2\n4\n");
+    assert_eq!(at.read("xaa"), "1\n3\n5\n");
+    assert_eq!(at.read("xab"), "2\n4\n");
+}
+
+#[test]
+#[cfg(any(target_os = "linux", target_os = "android"))]
+fn test_round_robin_limited_file_descriptors() {
+    new_ucmd!()
+        .args(&["-n", "r/40", "onehundredlines.txt"])
+        .limit(Resource::NOFILE, 9, 9)
+        .succeeds();
 }
 
 #[test]
@@ -1589,7 +1686,7 @@ fn test_split_invalid_input() {
         .args(&["-n", "0", "file"])
         .fails()
         .no_stdout()
-        .stderr_contains("split: invalid number of chunks: 0");
+        .stderr_contains("split: invalid number of chunks: '0'");
 }
 
 /// Test if there are invalid (non UTF-8) in the arguments - unix
@@ -1648,9 +1745,9 @@ fn test_split_separator_nl_lines() {
         .pipe_in("1\n2\n3\n4\n5\n")
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\n2\n");
-    assert_eq!(file_read(&at, "xab"), "3\n4\n");
-    assert_eq!(file_read(&at, "xac"), "5\n");
+    assert_eq!(at.read("xaa"), "1\n2\n");
+    assert_eq!(at.read("xab"), "3\n4\n");
+    assert_eq!(at.read("xac"), "5\n");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1661,9 +1758,9 @@ fn test_split_separator_nl_line_bytes() {
         .pipe_in("1\n2\n3\n4\n5\n")
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\n2\n");
-    assert_eq!(file_read(&at, "xab"), "3\n4\n");
-    assert_eq!(file_read(&at, "xac"), "5\n");
+    assert_eq!(at.read("xaa"), "1\n2\n");
+    assert_eq!(at.read("xab"), "3\n4\n");
+    assert_eq!(at.read("xac"), "5\n");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1673,9 +1770,9 @@ fn test_split_separator_nl_number_l() {
     ucmd.args(&["--number=l/3", "--separator=\n", "fivelines.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\n2\n");
-    assert_eq!(file_read(&at, "xab"), "3\n4\n");
-    assert_eq!(file_read(&at, "xac"), "5\n");
+    assert_eq!(at.read("xaa"), "1\n2\n");
+    assert_eq!(at.read("xab"), "3\n4\n");
+    assert_eq!(at.read("xac"), "5\n");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1685,9 +1782,9 @@ fn test_split_separator_nl_number_r() {
     ucmd.args(&["--number=r/3", "--separator", "\n", "fivelines.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\n4\n");
-    assert_eq!(file_read(&at, "xab"), "2\n5\n");
-    assert_eq!(file_read(&at, "xac"), "3\n");
+    assert_eq!(at.read("xaa"), "1\n4\n");
+    assert_eq!(at.read("xab"), "2\n5\n");
+    assert_eq!(at.read("xac"), "3\n");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1697,9 +1794,9 @@ fn test_split_separator_nul_lines() {
     ucmd.args(&["--lines=2", "-t", "\\0", "separator_nul.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\x002\0");
-    assert_eq!(file_read(&at, "xab"), "3\x004\0");
-    assert_eq!(file_read(&at, "xac"), "5\0");
+    assert_eq!(at.read("xaa"), "1\x002\0");
+    assert_eq!(at.read("xab"), "3\x004\0");
+    assert_eq!(at.read("xac"), "5\0");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1709,9 +1806,9 @@ fn test_split_separator_nul_line_bytes() {
     ucmd.args(&["--line-bytes=4", "-t", "\\0", "separator_nul.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\x002\0");
-    assert_eq!(file_read(&at, "xab"), "3\x004\0");
-    assert_eq!(file_read(&at, "xac"), "5\0");
+    assert_eq!(at.read("xaa"), "1\x002\0");
+    assert_eq!(at.read("xab"), "3\x004\0");
+    assert_eq!(at.read("xac"), "5\0");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1721,9 +1818,9 @@ fn test_split_separator_nul_number_l() {
     ucmd.args(&["--number=l/3", "--separator=\\0", "separator_nul.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\x002\0");
-    assert_eq!(file_read(&at, "xab"), "3\x004\0");
-    assert_eq!(file_read(&at, "xac"), "5\0");
+    assert_eq!(at.read("xaa"), "1\x002\0");
+    assert_eq!(at.read("xab"), "3\x004\0");
+    assert_eq!(at.read("xac"), "5\0");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1733,9 +1830,9 @@ fn test_split_separator_nul_number_r() {
     ucmd.args(&["--number=r/3", "--separator=\\0", "separator_nul.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1\x004\0");
-    assert_eq!(file_read(&at, "xab"), "2\x005\0");
-    assert_eq!(file_read(&at, "xac"), "3\0");
+    assert_eq!(at.read("xaa"), "1\x004\0");
+    assert_eq!(at.read("xab"), "2\x005\0");
+    assert_eq!(at.read("xac"), "3\0");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1745,9 +1842,9 @@ fn test_split_separator_semicolon_lines() {
     ucmd.args(&["--lines=2", "-t", ";", "separator_semicolon.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1;2;");
-    assert_eq!(file_read(&at, "xab"), "3;4;");
-    assert_eq!(file_read(&at, "xac"), "5;");
+    assert_eq!(at.read("xaa"), "1;2;");
+    assert_eq!(at.read("xab"), "3;4;");
+    assert_eq!(at.read("xac"), "5;");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1757,9 +1854,9 @@ fn test_split_separator_semicolon_line_bytes() {
     ucmd.args(&["--line-bytes=4", "-t", ";", "separator_semicolon.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1;2;");
-    assert_eq!(file_read(&at, "xab"), "3;4;");
-    assert_eq!(file_read(&at, "xac"), "5;");
+    assert_eq!(at.read("xaa"), "1;2;");
+    assert_eq!(at.read("xab"), "3;4;");
+    assert_eq!(at.read("xac"), "5;");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1769,9 +1866,9 @@ fn test_split_separator_semicolon_number_l() {
     ucmd.args(&["--number=l/3", "--separator=;", "separator_semicolon.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1;2;");
-    assert_eq!(file_read(&at, "xab"), "3;4;");
-    assert_eq!(file_read(&at, "xac"), "5;");
+    assert_eq!(at.read("xaa"), "1;2;");
+    assert_eq!(at.read("xab"), "3;4;");
+    assert_eq!(at.read("xac"), "5;");
     assert!(!at.plus("xad").exists());
 }
 
@@ -1781,9 +1878,9 @@ fn test_split_separator_semicolon_number_r() {
     ucmd.args(&["--number=r/3", "--separator=;", "separator_semicolon.txt"])
         .succeeds();
 
-    assert_eq!(file_read(&at, "xaa"), "1;4;");
-    assert_eq!(file_read(&at, "xab"), "2;5;");
-    assert_eq!(file_read(&at, "xac"), "3;");
+    assert_eq!(at.read("xaa"), "1;4;");
+    assert_eq!(at.read("xab"), "2;5;");
+    assert_eq!(at.read("xac"), "3;");
     assert!(!at.plus("xad").exists());
 }
 
