@@ -34,7 +34,10 @@ fn set_file_times(at: &AtPath, path: &str, atime: FileTime, mtime: FileTime) {
 
 fn str_to_filetime(format: &str, s: &str) -> FileTime {
     let tm = chrono::NaiveDateTime::parse_from_str(s, format).unwrap();
-    FileTime::from_unix_time(tm.and_utc().timestamp(), tm.timestamp_subsec_nanos())
+    FileTime::from_unix_time(
+        tm.and_utc().timestamp(),
+        tm.and_utc().timestamp_subsec_nanos(),
+    )
 }
 
 #[test]
@@ -189,7 +192,14 @@ fn test_touch_set_cymdhms_time() {
 
 #[test]
 fn test_touch_set_only_atime() {
-    let atime_args = ["-a", "--time=access", "--time=atime", "--time=use"];
+    let atime_args = [
+        "-a",
+        "--time=access",
+        "--time=atime",
+        "--time=atim", // spell-checker:disable-line
+        "--time=a",
+        "--time=use",
+    ];
     let file = "test_touch_set_only_atime";
 
     for atime_arg in atime_args {
@@ -290,7 +300,7 @@ fn test_touch_set_both_time_and_date() {
 
 #[test]
 fn test_touch_set_only_mtime() {
-    let mtime_args = ["-m", "--time=modify", "--time=mtime"];
+    let mtime_args = ["-m", "--time=modify", "--time=mtime", "--time=m"];
     let file = "test_touch_set_only_mtime";
 
     for mtime_arg in mtime_args {
@@ -679,10 +689,23 @@ fn test_touch_system_fails() {
 }
 
 #[test]
+#[cfg(not(target_os = "windows"))]
 fn test_touch_trailing_slash() {
     let (_at, mut ucmd) = at_and_ucmd!();
     let file = "no-file/";
-    ucmd.args(&[file]).fails();
+    ucmd.args(&[file]).fails().stderr_only(format!(
+        "touch: cannot touch '{file}': No such file or directory\n"
+    ));
+}
+
+#[test]
+#[cfg(target_os = "windows")]
+fn test_touch_trailing_slash_windows() {
+    let (_at, mut ucmd) = at_and_ucmd!();
+    let file = "no-file/";
+    ucmd.args(&[file]).fails().stderr_only(format!(
+        "touch: cannot touch '{file}': The filename, directory name, or volume label syntax is incorrect.\n"
+    ));
 }
 
 #[test]
@@ -698,7 +721,7 @@ fn test_touch_no_such_file_error_msg() {
 }
 
 #[test]
-#[cfg(not(target_os = "freebsd"))]
+#[cfg(not(any(target_os = "freebsd", target_os = "openbsd")))]
 fn test_touch_changes_time_of_file_in_stdout() {
     // command like: `touch - 1< ./c`
     // should change the timestamp of c
@@ -841,6 +864,7 @@ fn test_touch_no_dereference_dangling() {
 }
 
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_touch_dash() {
     let (_, mut ucmd) = at_and_ucmd!();
 
