@@ -18,6 +18,20 @@ fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
 }
 
+#[test]
+fn test_help_and_version_on_stdout() {
+    new_ucmd!()
+        .arg("--help")
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("Usage");
+    new_ucmd!()
+        .arg("--version")
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("uniq");
+}
+
 #[test]
 fn test_stdin_default() {
     new_ucmd!()
@@ -131,6 +145,21 @@ fn test_stdin_all_repeated() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("sorted-all-repeated.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=none"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=non"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=n"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated.expected");
 }
 
 #[test]
@@ -153,6 +182,16 @@ fn test_stdin_all_repeated_separate() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("sorted-all-repeated-separate.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=separat"]) // spell-checker:disable-line
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated-separate.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=s"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated-separate.expected");
 }
 
 #[test]
@@ -162,6 +201,16 @@ fn test_stdin_all_repeated_prepend() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("sorted-all-repeated-prepend.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=prepen"]) // spell-checker:disable-line
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated-prepend.expected");
+    new_ucmd!()
+        .args(&["--all-repeated=p"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("sorted-all-repeated-prepend.expected");
 }
 
 #[test]
@@ -239,6 +288,11 @@ fn test_group_prepend() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("group-prepend.expected");
+    new_ucmd!()
+        .args(&["--group=p"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("group-prepend.expected");
 }
 
 #[test]
@@ -248,6 +302,11 @@ fn test_group_append() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("group-append.expected");
+    new_ucmd!()
+        .args(&["--group=a"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("group-append.expected");
 }
 
 #[test]
@@ -257,6 +316,16 @@ fn test_group_both() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("group-both.expected");
+    new_ucmd!()
+        .args(&["--group=bot"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("group-both.expected");
+    new_ucmd!()
+        .args(&["--group=b"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("group-both.expected");
 }
 
 #[test]
@@ -266,6 +335,11 @@ fn test_group_separate() {
         .pipe_in_fixture(INPUT)
         .run()
         .stdout_is_fixture("group.expected");
+    new_ucmd!()
+        .args(&["--group=s"])
+        .pipe_in_fixture(INPUT)
+        .run()
+        .stdout_is_fixture("group.expected");
 }
 
 #[test]
@@ -283,6 +357,7 @@ struct TestCase {
 }
 
 #[test]
+#[allow(clippy::too_many_lines)]
 fn gnu_tests() {
     let cases = [
         TestCase {
